/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#define LMTP_H_RCSID "$Id: lmtp.h,v 1.125 2005/07/28 08:18:00 lwa Exp $"

#include <sys/types.h>
#include <sys/param.h>
#include <sys/time.h>

#include <regex.h>

#include <inttypes.h>
#include <unistd.h> 
#include <stdio.h>
#include <time.h>
#include <limits.h>

#include "mailaddr.h"
#include "re/re.h"
#include "libmdq/mdq.h"

#ifdef DMALLOC
#include <dmalloc.h>
#endif

#if WITH_DB
#include <db.h>
#endif

#if WITH_MYSQLLOG
#include <mysql.h>
#endif

/* defined below */
struct fstate;

/* terminating rule in filter */
#define FILTER_STOP      0
/* rule non terminating */
#define FILTER_CONTINUE  1
/* rule finishing the file parsing */
#define FILTER_ENDFILE   2
#define FILTER_ERROR    -1
#define FILTER_UNKNOWN  -2

#define STATE_INIT    0
#define STATE_CHAT    1
#define STATE_MAIL    2
#define STATE_RCPT    3
#define STATE_DATA    4
#define STATE_DELIVER 5
#define STATE_FILTER  6
#define STATE_FINAL   7

#ifndef CONFFILE
#define CONFFILE "/usr/local/etc/lmtpd.conf"
#endif

#ifndef PERL_FILTER
#define PERL_FILTER "/usr/local/etc/lmtpd-filter.pl"
#endif

#define ACTION_DELIVER  0x01
#define ACTION_FORWARD  0x02
#define ACTION_BOUNCE   0x04
#define ACTION_DROP     0x08
#define ACTION_ERROR    0x10

#define ACTION_LOG_ALL  0xff
#define ACTION_LOG_NONE 0

/* size of a line from the client in protocol   */
/* so a upper value for user names   :)         */
#define LINELEN 1024

#define FOLDER_TRASH "Trash"

#if HAVE_BOGUS_STRFIME_Z
#  define STRFTIME_FMT "%a, %e %b %Y %H:%M:%S (%Z)"
#else
#  define STRFTIME_FMT "%a, %e %b %Y %H:%M:%S %z (%Z)"
#endif
#define MID_FMT "%03lu"
#define ID_FMT "(" MID_FMT ") "

/* from procmail */

#define FROMDsubstitute "(^(\
Auto-Submitted:.*auto|\
Mailing-List:|Precedence:.*(junk|bulk|list)|\
To: Multiple recipients of |\
((Resent-)?(From|Sender)|X-Envelope-From|Return-Path):([^>]*[^(.%@a-z0-9])?(\
Post(ma?(st(e?r)?|n)|office)|(send)?Mail(er)?|daemon|m(mdf|ajordomo)|n?uucp|\
LIST(SERV|proc)|NETSERV|o(wner|ps)|r(e(quest|sponse)|oot)|b(ounce|bs\\.smtp)|\
echo|mirror|s(erv(ices?|er)|mtp(error)?|ystem)|\
A(dmin(istrator)?|MMGR|utoanswer)\
)(([^).!:a-z0-9][-_a-z0-9]*)?[%@>        ][^<)]*(\\(.*\\).*)?)?$([^>]|$)))"

/*
#define USELESS_ENDQUOTE "
*/

/* filter.c */
#define TAGMAX 16
#define ERRMSGMAX 256

#if !HAVE_ISBLANK
int isblank(int c);
#endif

#if HAVE_INTMAX_T && HAVE_STRTOIMAX
typedef intmax_t score_t;
#define A2S strtoimax
#define SCORE_FORMAT "%jd"
#define SCORE_MAX INTMAX_MAX
#elif HAVE_LONG_LONG && HAVE_STRTOLL
typedef long long score_t;
#define A2S strtoll
#define SCORE_MAX LLONG_MAX
#define SCORE_FORMAT "%lld"
#else
typedef long score_t;
#define A2S strtol
#define SCORE_FORMAT "%ld"
#define SCORE_MAX LONG_MAX
#endif

struct rcpt {
  char *user;
  char *mbox;
  char *path;
  char *ext;
  char *rcpt;
  char *bounce;
  uid_t uid;
  gid_t gid;
  int shared;
  unsigned long quota;
};

#define TRACE_ON       0x01
#define TRACE_DEFAULT  0x02

#define HEADER_PRIVATE              0

#define HEADER_SEND                -1
#define HEADER_OTHER               -1

#define HEADER_SUBJECT             -2
#define HEADER_RECEIVED            -3
#define HEADER_DELIVERED_TO        -4
#define HEADER_RETURN_PATH         -5
#define HEADER_MESSAGE_ID          -6
#define HEADER_CONTENT_TYPE        -7

#define HEADER_DATE                -8
#define HEADER_CONTENT_DISPOSITION -9
#define HEADER_FROM                -10
#define HEADER_IN_REPLY_TO         -11
#define HEADER_REFERENCES          -12
#define HEADER_TO                  -13
#define HEADER_CC                  -14
#define HEADER_BCC                 -15
#define HEADER_SENDER              -16

struct headers {
  char *header;
  char *utf8;
  unsigned char trace;     /* trace this header ? */
  int len;                 /* total length */
  int dd_len;              /* length of the header: part */
  int type;                /* HEADER_* or >= 0 for private headers*/
  uint32_t inset;
  struct headers *next;
};

struct mod_result {
  char *module;
  long result;
};

struct slist_elt {
  char *key;
  char *value;
  unsigned int flags;
  void *extra;
};

struct slist {
  int sorted;
  int nmemb;
  int (*compar)(const void *, const void *);
  int from_malloc;
  struct slist_elt *elts;
};

struct arg {
  int argc;
  char **argv;
};

struct regexp_list {
  regex_t preg;
  char *action;
  struct regexp_list *next;
};

struct rservers {
  int roll_percent;
  int current;
  int timeout;
  int fd;
  int keepalive;
  void (*close)(int);
  struct arg arg;
};

struct actionq {
  int level;
  int (*cmd)(struct fstate *, struct actionq *);
  char *arg;
  char action[ERRMSGMAX+1];
  unsigned char logmask;
  unsigned char logit;
  struct actionq *next;
};

#define ADDRESS_LEN sizeof("255.255.255.255")
#define TXT_RDATA_LEN 255
#define BL_QUERY_LEN 255

struct blresult {
  char *rbl;
  char query[BL_QUERY_LEN];
  int  proof;
  char *domain;
  char a[ADDRESS_LEN];
  char txt[TXT_RDATA_LEN];
  struct headers *header;
};

/* antivirus */

#define AV_DESCR_MAX 1024

typedef int (*av_scanner_t) (struct fstate *state);

struct antivirus {
  char *type;
  int nvirus;
  int scanned;
  char *desc;
  char bounce_message[50];
  char virus[50];
  int (*scan)(struct fstate *);
  struct rservers drweb;
  struct rservers sophie;
  struct rservers clamd;
  int clamd_mbox_format;
  struct rcpt viruskeeper;
  struct timeval start_time;
  struct slist action;
};

/* state of program */

struct fstate {
  /* filter */
  score_t score;
  char tag[TAGMAX+1];
  char *trace;
  unsigned char action_log;
  char *vacation;
  char *user_bounce_message;
  char *default_dest;  /* default mailbox or forward address */
  char *timezone_user;
  struct actionq *actions;
  int hlevel; 
  int logged_message;
  FILE *filter_debug;
  char *charset;

  /* delivery */
  char error[ERRMSGMAX+1];
  char *qrecipient;
  struct rcpt *rcpt;
  MDQ *quotainfo;
  char *syslog_host;
  struct timeval date_deliver;
  int forwards;
  int spamassassin_done;
  double spamassassin_score;
  int spamassassin_is_spam;

  /* storage */
  FILE *spool;
  struct headers *hdrlist;
  struct headers *hdr_delivered;
  struct headers *hdr_my_received;
  struct headers *hdr_in_received;
  char *messageid;     /* Message-ID header */
  size_t msgsize;
  size_t bodysize;
  char *body;
  long nblines;
  long nbheaders;
#if 0
  struct mod_result *module_cache;
  int module_cache_size;
#endif
  struct slist mod_cache;
  char *mod_message;
  int in_module;

  /* rcpt to */
  struct rcpt *rcpts;
  int nrcpts;          /* size of rcpts */
  int nrcpts_r;        /* number of rcpt to tried */

  /* mail from */
  char *sender;
  time_t date;
  int can_bounce;

  /* control */
  unsigned long counter;
  int state;
  char *lhlo;
  pid_t pid;
  time_t date_start;
  int timeout_code;    /* error code to return on timeout */
  int fatal;
  char *sasl_user;
  char *sasl_method;

  /* config */
  char *myname;
  char *share_map;
  char *share_domain;
  char *herhostname;
  char *myhostname;
  char *logname;
  struct arg filtername;
  char *vacationcache;
  int vacationcache_create;
  int max_Received;
  regex_t *received_front;
  int timeout_cmd;
  int timeout_dataline;
  int timeout_deliver;
  struct rcpt spool_user;
  struct rcpt missing_user;
  mode_t mode_hier;
  mode_t mode_mail;
  unsigned long max_size;
  char *timezone;
  char *module_path;
  int bodycheck_maxlen;
  struct slist quota_timefatal_acl;
  struct slist syslog_acl;
  struct slist rbl_server;
  struct slist rhsbl_server;
  struct slist sasl_users;
  struct slist filter_debug_msg;
  int sasl_need_auth;
  int sasl_noplain;
  int res_retry;
  int res_retrans;
  char *bounce_message;
  char *localdomain;
  int no_domain_recipient;
  char *quota_warnmsg;
  char *quota_mark;
  int quota_timefatal;
  int quota_db_timeout;
  int quota_maxchecks;
  char *quota_db_method;
  int forwards_max;
  char *sendmail_path;
#if WITH_DB
  DB     *quota_db;
  DB_ENV *quota_dbenv;
  char *quota_db_dbfile;
  char *quota_db_dbenv;
#endif
  char *quota_db_linkdir;
#if WITH_MYSQLLOG
  MYSQL quota_db_mysql;
  int quota_db_mysql_opened;
  char *quota_db_mysql_host;
  char *quota_db_mysql_user;
  char *quota_db_mysql_password;
  char *quota_db_mysql_database;
  char *quota_db_mysql_table;
  char *quota_db_mysql_col_user;
  char *quota_db_mysql_col_code;
  char *quota_db_mysql_col_time;
  char *quota_db_mysql_col_num;
#endif
  char recipient_delimiter;
  int syslog_facility;
  time_t log_mintime;
  unsigned long log_maxsize;
  char *log_msg;
#if WITH_MYSQLLOG
  MYSQL mysqllog;
  int   mysqllog_opened;
  char  *mysqllog_host;
  char  *mysqllog_user;
  char  *mysqllog_password;
  char  *mysqllog_database;
  char  *mysqllog_query;
#endif
  char *mailinglists_file;
  int includes; /* level of include in configuration file, internal use */
  struct antivirus av;
  char *av_default_header;
  int av_checkall;
  struct rservers spamassassin;
  struct slist spamassassin_acl;
  int spamassassin_maxsize;
  struct slist pipe_path;
  struct sockaddr_in *syslog_src;
  int mybounce;
  struct arg mybounce_msg;
  char *mybounce_path;

  char *perl_filtername;
  char *perl_eval;
  int perl;

  regex_t  *alias_format;
  int       alias_sub_addresses;
  char     *alias_file;

  int filesystem_change_atime_when_write;
  int filesystem_with_noatime;
  struct regexp_list *file_charset_re;

  int reply_id;

#ifdef UNUSED_ACTION_LOG
  int action_log_deliver_facility;
  int action_log_forward_facility;
  int action_log_bounce_facility;
  int action_log_drop_facility;
  int action_log_error_facility;

  char *action_log_deliver_format;
  char *action_log_forward_format;
  char *action_log_bounce_format;
  char *action_log_drop_format;
  char *action_log_error_format;
#endif

};

#ifndef __GNUC__
#define __attribute__(x) 
#endif


/*   retourne l'entete et sa longueur
 *   "" (contant) si fin d'entete
 *  NULL si fin de fichier ou erreur
 */

char *readheader(FILE *f, int *len, int *dd_len);
/* change/cre un entete, retourne NULL en cas d'erreur */
struct headers *setheader(int type, char *fmt, ...)
     __attribute__((format(printf, 2, 3)));

/*
 * store.c
 */

char *store (struct fstate*);
char *prepare (struct fstate*);

/*
 * deliver.c
 */

char *deliver(struct fstate *state, struct rcpt *rcpt1);
int setuidgid(struct fstate *state, struct rcpt *rcpt1);

/*
 * filter.c
 */

struct cmd {
  char *name;
  size_t len;
  int (*f)(struct fstate*, char*);
};

char *filter (struct fstate*);
score_t readscore(struct fstate *state, char **buf);
int filterfile_cmd(struct fstate *state, char *file, struct cmd *cmds,
	       int (*cmd_unknown)(struct fstate *state, char *, char *));
int filterfile(struct fstate *state, char *file);

char *readlogtype(struct fstate *state, char *s);

/*
 * chk.c
 */

int regchk(RE *re, char *hdr, int sense, struct headers *hdrlist, int type);
int bodchk(struct fstate *state, RE *re, int len);
int valid_re(RE *re);
int loopchk(char *rcpt, struct headers *hdrlist);
char *mod_run(struct fstate *state, char *module);
int modchk(struct fstate *state, RE *re, char *module, int sense);
char *utf8_header(struct headers *h);

/*
 * cmd.c
 */

int cmd_deliver(struct fstate*, char *);
int cmd_drop(struct fstate*, char *);
int cmd_bounce(struct fstate*, char *);
int cmd_forward(struct fstate*, char *);
int cmd_forward2(struct fstate*, char *);
int cmd_score(struct fstate*, char *);
int cmd_skipto(struct fstate*, char *);
int cmd_trash(struct fstate*, char *);
int unknown_cmd(struct fstate*, char *, char *);
int default_cmd(struct fstate*);
int cmd_vacation(struct fstate *state, char *a);
int cmd_logrotate(struct fstate *state, char *a);
int cmd_trace(struct fstate *state, char *a);
int cmd_timezone(struct fstate *state, char *a);
#define MAX_USER_HEADER_LEN 1024
int cmd_add(struct fstate *state, char *a);
int cmd_blacklist(struct fstate *state, char *a);
int cmd_extension(struct fstate *state, char *a);
int cmd_stow(struct fstate *state, char *a);
int cmd_log(struct fstate *state, char *a);
int cmd_syslog(struct fstate *state, char *a);
int cmd_deliver2(struct fstate*, char *);
int cmd_default(struct fstate *state, char *a);
int cmd_antivirus(struct fstate *state, char *a);
int cmd_spamassassin(struct fstate *state, char *a);
int cmd_header_set(struct fstate *state, char *a);
int cmd_moved(struct fstate *state, char *a);
int cmd_pipe(struct fstate *state, char *a);
int cmd_pipe2(struct fstate *state, char *a);
int cmd_debug(struct fstate *state, char *a);
int cmd_stop(struct fstate *state, char *a);
int cmd_archive(struct fstate *state, char *a);
int cmd_archive2(struct fstate *state, char *a);

/*
 * action.c
 */

int push_action(struct fstate *state, char *a, 
		int(*cmd)(struct fstate*, struct actionq*));
int push_action_log(struct fstate *state, char *fmt, ...);
void free_action(struct fstate *state);
int flush_action(struct fstate *state);

int action_deliver(struct fstate *state, struct actionq *act);
int action_deliver0(struct fstate *state, struct actionq *act);
int action_forward(struct fstate *state, struct actionq *act);
int action_forward2(struct fstate *state, struct actionq *act);
int action_bounce(struct fstate *state, struct actionq *act);
int action_moved(struct fstate *state, struct actionq *act);
int action_trash(struct fstate *state, struct actionq *act);
int action_drop(struct fstate *state, struct actionq *act);
int action_pipe(struct fstate *state, struct actionq *act);
int action_pipe2(struct fstate *state, struct actionq *act);
int action_unknown(struct fstate *state, struct actionq *act);

int action_av_notify(struct fstate *state, struct actionq *act);

#if 0
/*
 * module.c
 */
int cmd_module(struct fstate *state, char *a);
int cmd_return(struct fstate *state, char *a);
void free_mod_cache(struct mod_result *cache, int ncache);
#endif

/*
 * mod.c
 */

int cmd_mod(struct fstate *state, char *a);
int cmd_message(struct fstate *state, char *a);
int cmd_ok(struct fstate *state, char *a);


/*
 * state.c
 */

int resetstate(struct fstate *state, int new);
int closestate(struct fstate *state);

/*
 *  send.c
 */

char *sendmd(struct fstate *state, char *box);
int runsendmail(struct fstate *state, char *sendmail, 
		char *sender, char *destination);
char *waitsendmail(struct fstate *state, int fd);
char *sendmail(struct fstate *state, char *, char*);
char *send_vacation(struct fstate *state, char *message);
ssize_t mailwrite(struct fstate *state, struct headers *hdrlist,
		  int level, int fd);
ssize_t mailwrite_size(struct fstate *state, struct headers *hdrlist,
		       int level);
char *send_pipe(struct fstate *state, char *box, char *argv[], char *path);

/*
 * error.c
 */

#define MemoryError(state) seterr(state, 421, "4.3.0", "memory exhausted")
#define DeliveryError(state, msg) seterr(state, 450, "4.2.0", "%s: %s", msg, strerror(errno))
#define DeliverySysError(state, msg) seterr(state, 450, "4.3.0", "%s: %s", msg, strerror(errno))

#define OkNoError(state) seterr(state, 250, "2.1.5", "mail for <%s> delivered", state ? state->qrecipient : "")

char *seterr(struct fstate *state, int, char*, char *fmt, ...)
#if __FreeBSD_cc_version > 300000
     __attribute__ ((format (printf0, 4, 5)))
#endif
;

/*
 *  log.c
 */

int logit(struct fstate *state, struct actionq *act,
	  unsigned char type, char *fmt, ...)
     __attribute__ ((format (printf, 4, 5)));
int logrotate(struct fstate *state);
int logflush(struct fstate *state, struct actionq *act);
/*
 *  box.c
 */

char *getbox(char*);

/*
 * lmtpconf.c
 */

#define MAX_INCLUDE_LEVEL 20
int readconf(char *file, struct fstate *state);

/*
 * checkrelay.c
 */ 

struct blresult * checkbl(struct fstate *state, char *rbl);
int get_addr_from_received(char *str, int ip[4],
                           char *query, char *a, char *txt);
/*
 * exit.c
 */

void setsignals(void);
char *exiting(struct fstate *state);
void timeout(struct fstate *state, int t, int code);

/*
 * quotawarn.c
 */

int quotawarn(struct fstate *state, struct rcpt *rcpt1);

#if WITH_DB
void quotadb_db3_add(struct fstate *state, char *user, int code);
void quotadb_db3_del(struct fstate *state, char *user);
char *quotadb_db3_check(struct fstate *state, char *user);
int quotadb_db3_open(struct fstate *state);
void quotadb_db3_close(struct fstate *state);
#endif

void quotadb_link_add(struct fstate *state, char *user, int code);
void quotadb_link_del(struct fstate *state, char *user);
char *quotadb_link_check(struct fstate *state, char *user);
int quotadb_link_open(struct fstate *state);
void quotadb_link_close(struct fstate *state);

#if WITH_MYSQLLOG
void quotadb_mysql_add(struct fstate *state, char *user, int code);
void quotadb_mysql_del(struct fstate *state, char *user);
char *quotadb_mysql_check(struct fstate *state, char *user);
int quotadb_mysql_open(struct fstate *state);
void quotadb_mysql_close(struct fstate *state);

void quotadb_mysqladd_add(struct fstate *state, char *user, int code);
int quotadb_mysqladd_open(struct fstate *state);
#endif

void (*quotadb_add)(struct fstate *state, char *user, int code);
void (*quotadb_del)(struct fstate *state, char *user);
char *(*quotadb_check)(struct fstate *state, char *user);
int quotadb_open(struct fstate *state);
void (*quotadb_close)(struct fstate *state);

/*
 * mysqllog.c 
 */

#if WITH_MYSQLLOG
int mysqllog(struct fstate *state, char *buf);
void mysqlcloselog(struct fstate *state);
#endif

/*
 *  user.c
 */

void freercpt1(struct rcpt *rcpt1);
struct rcpt *getrcpt(struct fstate *state, struct rcpt *rcpt1);
char *newrcpt(struct fstate *state, char *user);
void do_vrfy(struct fstate *state, char *user);
void do_expn(struct fstate *state, char *user);

/*
 * tools.c
 */

int opentmp(struct fstate *, char tmpname[PATH_MAX]);
char *getuserdomain(char *user, char domain[255]);
FILE *opentemplate(char *path, char *domain);
FILE *newmessage(struct fstate *state, char *type, char tmpname[PATH_MAX]);
int endmessage(struct fstate *state, char *type, char *tmpname, FILE *tmp);
char *lower(char *s);
void mkbox(struct fstate *state, char *box);
char *purify_header(char *src, char *content, int len);
#if !HAVE_VSYSLOG
void vsyslog(int priority, const char *message, va_list args);
#endif
int split_args(char *cmd, char *args, size_t arglen,
	       char *argv[], int argcmax);


/*
 * time.c
 */

void time_init(struct fstate *state);
struct tm *usertime(struct fstate *state);
score_t scoretime(struct fstate *state, char **buf);
long a2delay(char *a);

/*
 * slist.c
 */

struct slist_elt *slist_search(struct slist *m, char *key);
struct slist *slist_put(struct slist *m, char *key, char *value);
void slist_clean(struct slist *m);
void slist_dump(struct slist *m, FILE *f);
void slist_reset(struct slist *m);
int slist_casecmp(const void *a, const void *b);
int slist_cmp(const void *a, const void *b);
char *slist_get_value(struct slist *m, char *key);


/*
 * strfmt.c
 */

char *strfmt(char *buf, int buflen, char *fmt, char *val, ...);

/*
 * isheader.c
 */

char *isheader(char *s);


/*
 * checker.c
 */

#ifdef PARSER
void parser_nextline(char *buf);
#endif

/*
 * my_syslog.h
 */

int my_syslog_check_addr(struct fstate *state, int af, void *src);
int my_syslog_check_name(struct fstate *state, char *name);
int my_syslog_check_user(struct fstate *state, char *user);
int my_syslog_check(struct fstate *state, char *host);
void my_syslog(struct fstate *state, char *fmt, ...)
     __attribute__ ((format (printf, 2, 3)));

/*
 * user_res.c
 */
struct user_res_ctx {
  int res_retry;
  int res_retrans;
};
int user_res_begin(struct fstate *state, struct user_res_ctx *ctx);
int user_res_end(struct fstate *state, struct user_res_ctx *ctx);

/*
 * special.c
 */

RE * special_re(struct fstate *state, char **buf);

/*
 * version.c
 */

void version(void);

/*
 * auth.c
 */

char *auth(struct fstate *state, char *method, char *param);

/*
 * xconnect.c
 */

int xconnect(char *path, int timeout);
void rxinit(struct rservers *servers, int keepalive, void (*close_f)(int));
int rxconnect(struct rservers *paths);
int rxconnect_test(struct rservers *servers, char *buf, int len);
int rxclose(struct rservers *paths);
int roll_server(struct rservers *servers, char *reason, ...)
#if __FreeBSD_cc_version > 300000
     __attribute__ ((format (printf0, 2, 3)))
#endif
;
char *current_server(struct rservers *servers);

/*
 * spam-assassin.c
 */

void spam_assassin_init(struct fstate *state);
int spam_assassin(struct fstate *state, char *a);

/*
 * av.c
 */

int av_init(struct fstate *state);
int av_scan(struct fstate *state);
void av_reset(struct fstate *state);
void av_set_virus(struct fstate *state, char *antivirus, char *virus);
int av_message(struct fstate *state, char *path);

/*
 * av-drweb.c
 */

av_scanner_t av_drweb_init(struct fstate *state);

/*
 * av-sophie.c
 */

av_scanner_t av_sophie_init(struct fstate *state);

/*
 * av-clamd.c
 */

av_scanner_t av_clamd_init(struct fstate *state);

/*
 * sieve.c
 */
int filtersieve(struct fstate *state, char *file);

/*
 * mybounce.c
 */

int mybounce(struct fstate *state);
int mybounce_init(struct fstate *state);

/*
 * score.c
 */
/* unsigned int nb_headers(char **hdr, struct headers *hdrlist); */
int test_scalar(struct fstate *state, char **buf);
score_t readscore(struct fstate *state, char **buf);

/*
 * perl.c
 */

#if WITH_PERL
int fperl_init(struct fstate *state);
int filterperl(struct fstate *state, char *file);
#endif

/*
 * set.c
 */

uint32_t get_set(char a);

/*
 * acl.c
 */

char *acl_mailbox(struct slist *acl, char *key);
char *find_relist(struct regexp_list *rl, char *key);

/*
 * debug.c
 */

FILE *debug_init(struct fstate *state);
int debug_close(struct fstate *state, FILE *f);

#if __STDC_VERSION__ >= 199901L || __GNUC__ >= 3
#define fdebug(state, ...) if (state->filter_debug) \
     fprintf(state->filter_debug,  __VA_ARGS__)
#elif __GNUC__ <= 2
#define fdebug(state, x...) if (state->filter_debug) \
     fprintf(state->filter_debug,  x)
#else
#error Need C99 or GCC
#endif
