#! /usr/bin/perl
#
#    Copyright 2000, 2001, 2002, 2003, 2004 Laurent Wacrenier
#
#    This file is part of PLL lmtpd.
#
#    PLL lmtpd is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    PLL lmtpd is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PLL lmtpd; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: lmtpdlogsum.pl,v 1.8 2004/05/21 13:28:44 lwa Exp $
use strict;
use Getopt::Long;

my %connect;
my %from_d;
my %to_d;
my %from;
my %to;
my $nlist=20;
my $session;
my %reject;
my %rejectreason;
my %virus;
my %virus_sender;
my @time;
my %total_action;

my $totalsize=0;
my $totalfrom=0;
my $totalto=0;
my $totalrej=0;

my $alloweddomains = '^.*$';
my $alloweddomainsmatch = '.*';

my %options;

Getopt::Long::Configure("pass_through");
if (!GetOptions(\%options, "config-file=s","help")) {
  exit 1;
}

if (defined($options{"help"})) {
    print STDERR <<EOT;
Usage : lmtpdlogsum.pl --config-file=<file>
                       --nb-senders=<nb> --nb-recipients=<nb>
                       --nb-sender-size=<nb> --nb-recipient-size=<nb>
                       --nb-sender-domain=<nb> --nb-reject=<nb>
                       --nb-recipient-domain=<nb> --domain=<domain>
		       --nb-recipient-size=<nb>
		       --nb-recipient-time=<nb>
		       --nb-virus-sender=<nb>
Set option to -1 to see full listing
EOT
  exit 0;
}

if (defined(my $configfile=$options{"config-file"})) {
  open CFFILE, "<$configfile" or die "Can not open $configfile : $!"; 
  while (defined(my $line=<CFFILE>)) {
    chomp $line;
    push @ARGV,$line;
  }
  close CFFILE;
}

Getopt::Long::Configure("default");
%options=();
if (!GetOptions(\%options,
		"nb-senders=i",
		"nb-recipients=i", 
		"nb-sender-size=i",
		"nb-recipient-size=i",
		"nb-recipient-time=i",
		"nb-sender-domain=i", 
		"nb-reject=i",
		"nb-recipient-domain=i",
		"nb-virus-sender=i",
		"domain=s")) {
    exit 1;
}

my $nbsend=$options{"nb-senders"} || $nlist;
my $nbrcpt=$options{"nb-recipients"} || $nlist;
my $nbsentsize=$options{"nb-sender-size"} || $nlist;
my $nbrcptsize=$options{"nb-recipient-size"} || $nlist;
my $nbsentdom=$options{"nb-sender-domain"} || $nlist;
my $nbrcptdom=$options{"nb-recipient-domain"} || $nlist;
my $nbrej=$options{"nb-reject"} || $nlist;
my $nbrcpttime = $options{"nb-recipient-time"} || $nlist;
my $nb_virus_sender=$options{"nb-virus-sender"} || $nlist;

if (defined($options{"domain"})) {
  $alloweddomains= '^' . $options{"domain"} . '$';
  $alloweddomainsmatch = $options{"domain"};
}

#Jan  2 00:03:05 pop lmtpd[65715]: (00015)
while(<STDIN>) {
    chomp;
    my ($date, $pid, , $lpid, $str) =
	/^(...\s+[0-9]+\s..:..:..)\s\S+\slmtpd\[(\d+)\]:\s*(?:\((\d+)\)\s*)?(.*)/;
    if ($str =~ /^connect from (\S+)/) {
	$connect{$1}++;
    } elsif ($str =~ /^from=<(.*)>, size=(\d+), nrcpt=(\d+)/) {
	my $sender = $1;
	my $size   = $2;
	my $nrcpt  = $3;
        $session->{$pid}->{$lpid}=[ $size, $nrcpt , $sender ];
    } elsif ($str =~ /^to=<(.*)>, (mbox='(.*)', )?(delay=(\S+), )?action=(.*)/) {
	my $to     = $1;
	my $mbox   = $3;
	my $delay  = $5;
	my $action = $6;
	my $filtered = $action ne "delivered";
	my ($domain) = ($to =~ /@([^@]*)$/);
	$domain = lc $domain;
        next unless ($domain  =~ /$alloweddomains/io);

        my $sender = $session->{$pid}->{$lpid}->[2];
        my $size = $session->{$pid}->{$lpid}->[0];
        my ($senderdomain) = ($sender =~ /@([^@]*)$/);
	$senderdomain = lc $senderdomain;

	$from{$sender}=[$from{$sender}->[0] + $size,
			$from{$sender}->[1] + 1,
			$from{$sender}->[2] + 1 ];
	$from_d{$senderdomain}=[$from_d{$senderdomain}->[0] + $size,
			  $from_d{$senderdomain}->[1] + 1,
			  $from_d{$senderdomain}->[2] + 1 ];
 
	$to{$to} = [ $to{$to}->[0] + $session->{$pid}->{$lpid}->[0],
		     $to{$to}->[1] + 1,
		     $to{$to}->[2] + $filtered,
		     $to{$to}->[3] + $delay ];

	$to_d{$domain} = [$to_d{$domain}->[0] + $size*1.0,
			  $to_d{$domain}->[1] + 1,
			  $to_d{$domain}->[2] + $filtered,
			  $to_d{$domain}->[3] + $delay ];
        $totalto++;
        $totalsize+=$size*1.0;
	my ($cmd_action) = $action =~ /(\S+)/;
	$total_action{$cmd_action}->[0] += 1;
	$total_action{$cmd_action}->[1] += $size*1.0;
    	my ($hour) = ($date =~ /(..):..:..$/);
	$time[$hour] = $time[$hour] + 1;
        $totalfrom++;

    } elsif ($str =~ /(reject|delay|fatal):\s*(.*)/) {
	my $cause = $2;
        next if $cause =~ /503 5.5.1 no recipients; DATA/i;
        next unless $cause =~ /$alloweddomainsmatch/io;
	$cause =~ s/\s*;.*//;
	$reject{$cause}++;
        $totalrej++;
    } elsif ($str =~ /\[\S+\] infected with (\S+)(?: \((?:scan_time=(\S+)s)(?:, smtp_client=(\S+)\)))?/) {
	my $size = $session->{$pid}->{$lpid}->[0];
	$virus{$1}->{count}++;
	$virus{$1}->{size} += $size;
	$virus{$1}->{scan_time} += $2;
	if (defined $3) {
	    $virus_sender{$3}->{count}++;
	    $virus_sender{$3}->{size}+=$size;
	    $virus_sender{$3}->{scan_time}+=$2;
	}
    }
    next if $str eq "";
#    print "$lpid $str\n";
}


print "LMTP connection sources\n   ";
print "-" x 50, "\n";
for (keys %connect) {
    printf "%5d %s\n", $connect{$_}, $_;
}
print "\n";

my $n=$nbsentsize;
print "Top $n " if $n > 0;
print "Mail Senders by size\n";
print "          Size  Rcpts   Msgs Sender\n   ", "-"x50, "\n";
for (sort { $from{$b}->[0] <=> $from{$a}->[0] } keys %from) {
    printf "   %11d %6d %6d <%s>\n",
    $from{$_}->[0], $from{$_}->[1], $from{$_}->[2], $_;
    last if --$n == 0;
}
print "\n";

my $n=$nbsend;
print "Top $n " if $n > 0;
print "Mail Senders by recipient number\n";
print "          Size  Rcpts   Msgs Sender\n   ", "-"x50, "\n";
for (sort { $from{$b}->[1] <=> $from{$a}->[1] } keys %from) {
    printf "   %11d %6d %6d <%s>\n",
    $from{$_}->[0], $from{$_}->[1], $from{$_}->[2], $_;
    last if --$n == 0;
}
print "\n";

my $n=$nbsentdom;
print "Top $n " if $n > 0;
print "Mail Domain Senders by message number\n";
print "          Size  Rcpts   Msgs Domain\n   ", "-"x50, "\n";
for (sort { $from_d{$b}->[2] <=> $from_d{$a}->[2] } keys %from_d) {
    printf "   %11d %6d %6d %s\n",
    $from_d{$_}->[0], $from_d{$_}->[1], $from_d{$_}->[2], $_;
    last if --$n == 0;
}
print "\n";

my $n=$nbrcptsize;
print "Top $n " if $n > 0;
print "Mail Recipient by size\n";
print "          Size    Msg    Time Delay Filter Recipient\n   ","-"x50, "\n";
for (sort { $to{$b}->[0] <=> $to{$a}->[0] } keys %to) {
    printf "   %11d %6d %7.2f %5.2f %6d <%s>\n",
    $to{$_}->[0], $to{$_}->[1], $to{$_}->[3], 
    $to{$_}->[3]/$to{$_}->[1], $to{$_}->[2], $_;
    last if --$n == 0;
}
print "\n";

my $n=$nbrcpt;
print "Top $n " if $n > 0;
print "Mail Recipient by message number\n";
print "          Size    Msg    Time Delay Filter Recipient\n   ","-"x50, "\n";
for (sort { $to{$b}->[1] <=> $to{$a}->[1] } keys %to) {
    printf "   %11d %6d %7.2f %5.2f %6d <%s>\n",
    $to{$_}->[0], $to{$_}->[1], $to{$_}->[3], 
    $to{$_}->[3]/$to{$_}->[1], $to{$_}->[2], $_;
    last if --$n == 0;
}
print "\n";

my $n=$nbrcpttime;
print "Top $n " if $n > 0;
print "Mail Recipient by time spend\n";
print "          Size    Msg    Time Delay Filter Recipient\n   ","-"x50, "\n";
for (sort { $to{$b}->[3] <=> $to{$a}->[3] } keys %to) {
    printf "   %11d %6d %7.2f %5.2f %6d <%s>\n",
    $to{$_}->[0], $to{$_}->[1], $to{$_}->[3], $to{$_}->[3]/$to{$_}->[1], $to{$_}->[2], $_;
    last if --$n == 0;
}
print "\n";

my $n=$nbrcptdom;
print "Top $n " if $n > 0;
print "Mail Domain Recipient by message number\n";
print "              Size     Msg Delay Filter Domain\n   ", "-"x50, "\n";
for (sort { $to_d{$b}->[1] <=> $to_d{$a}->[1] } keys %to_d) {
    printf "   %9.2f Mbytes %6d %5.2f %6d %s\n",
    $to_d{$_}->[0]/(1024*1024), $to_d{$_}->[1], $to_d{$_}->[3]/$to_d{$_}->[1],
    $to_d{$_}->[2], $_;
    last if --$n == 0;
}
print "\n";

print "Total by actions\n";
print "-" x 50, "\n";
print "                  Size            Msg\n";
for (sort keys %total_action) {
    printf " %-10s  %9.2f Mbytes  %6d\n",
    $_, $total_action{$_}->[1]/(1024*1024), $total_action{$_}->[0]
}
print "\n";

my $n=$nbrej;
print "Top $n " if $n > 0;
print "Rejected commands\n   ";
print "-" x 50, "\n";
for (sort { $reject{$b} <=> $reject{$a} } keys %reject ) {
    my $cause = $_;
    $cause =~ s/<[^>]+>:* //;
    $rejectreason{$cause}+=$reject{$_};
    next if --$n <= 0;
    printf "   %6d   %s\n",
    $reject{$_}, $_;
}
print "\n";

print "Detected Virus\n";
print "   Count      Size             Time Name\n   ", "-"x50, "\n";;
my $viruses = 0;
my $viruses_size = 0;
my $virus_time = 0;
for (sort { $virus{$b}->{count} <=> $virus{$a}->{count}} keys %virus) {
    printf "  %6d %9.2f Mbytes  %8.2f %s\n", $virus{$_}->{count},
    $virus{$_}->{size}/(1024*1024),
    $virus{$_}->{scan_time},
    $_;
    $viruses += $virus{$_}->{count};
    $viruses_size += $virus{$_}->{size};
    $virus_time += $virus{$_}->{scan_time};
}
print "    ----     -----          -----  -------\n";
printf "  %6d %9.2f Mbytes  %8.2f %s\n", $viruses, $viruses_size/(1024*1024),
    $virus_time,  "TOTAL";
print "\n";


my $n=$nbrcptdom;
print "Top $n SMTP servers sending virus\n";
print "   Count      Size             Time Sender IP address\n   ", "-"x50, "\n";;
for (sort { $virus_sender{$b}->{count} <=> $virus_sender{$a}->{count}} keys %virus_sender) {
    printf "  %6d %9.2f Mbytes  %8.2f %s\n", $virus_sender{$_}->{count},
    $virus_sender{$_}->{size}/(1024*1024),
    $virus_sender{$_}->{scan_time},
    $_;
    last if --$n == 0;
}
print "\n";


print "Rejected Causes\n   ";
print "-" x 50, "\n";
for (sort { $rejectreason{$b} <=> $rejectreason{$a} } keys %rejectreason ) {
    printf "   %6d   %s\n",
    $rejectreason{$_}, $_;
}
print "\n";


print "Per hour traffic\n";
my $max = 1;
for (0..23) {
    $max = $time[$_] if $max < $time[$_];
}
print "        time    Msg \n   ", "-"x50, "\n";
for (0..23) {
    printf "   %02d00-%02d00 %6d %s\n", $_, $_+1, $time[$_],
    "#" x ($time[$_]*50/$max);
}

printf "\n  A # represents %d messages\n", $max/50;

print "\n";
print "Grand totals\n   ";
print "-" x 50, "\n";
printf "Total Sent       : %6d\n",$totalfrom;
printf "Total Delivered  : %6d\n",$totalto;
printf "Total Size       : %9.2f Mbytes\n",$totalsize/(1024*1024);
printf "Total Rejected   : %6d\n",$totalrej;
print "\n\n";

