/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED = 
"$Id: re-system.c,v 1.8 2004/10/26 09:59:21 lwa Exp $";



#include <sys/types.h>

#include <stdlib.h>
#include <regex.h>

#include "re.h"

static char re_error[100];


static void re_close(RE *re) {
  if (re->code) {
    regfree(re->code);
    free(re->code);
  }
  re->code = NULL;
}

static int re_compile(RE *re, char *pattern, int icase) {
  int ret;
  re->code = malloc(sizeof(regex_t));
  if (re->code==NULL) {
    re->err = RE_ERR_MEMORY;
    re->errstr = "memory";
  } else {
    ret = regcomp(re->code, pattern, re->options |(icase?REG_ICASE:0));
    if (ret != 0) {
      regerror(ret, re->code, re_error, sizeof(re_error)-1);
      re->errstr = re_error;
      if (ret == REG_ESPACE) {
	re->err =  RE_ERR_MEMORY;
      } else {
	re->err =  RE_ERR_SYNTAX;
      }
    } else {
      re->errstr = NULL;
      re->err = 0;
    }
  }
  return re->err;
}

static int re_exec(RE *re, char *string) {
  int ret = regexec(re->code, string, 0, NULL, 0);
  return ret!=0;
}

RE re_extended = {
  re_compile,
  re_exec,
  re_close,
  REG_EXTENDED|REG_NOSUB,
  0,
};

RE re_basic = {
  re_compile,
  re_exec,
  re_close,
  REG_NOSUB,
  0,
};

#ifdef REG_NOSPEC
RE re_nospec = {
  re_compile,
  re_exec,
  re_close,
  REG_NOSPEC|REG_NOSUB,
  0,
};
#endif
