/*
   Copyright 2005 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include "config.h"

static char const rcsid[] UNUSED =
"$Id: sieve-check.c,v 1.1 2005/07/28 08:18:01 lwa Exp $";

#include "sieve.h"

#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>

int valid_folder(void *message, const char *folder) {
  return 1;
}

unsigned spamtest(void *message) {
  return 0;
}

unsigned virustest(void *message) {
  return 0;
}

char *from(void *message) {
  return "";
}

char *to(void *message) {
  return "";
}

int file_fgetc(void *f) {
  FILE *ff;
  return fgetc(ff);
}

size_t size(void *message) {
  return 0;
}

int main(int argc, char **argv) {
  int allow_fileinto = 1;
  int allow_reject = 1;
  int allow_enveloppe = 1;
  int allow_spamtest = 1;
  int allow_virustest  = 1;
  int allow_subadress  = 1;
  struct sieve_methods Pmethods = {0,};
  struct sieve_result Presult = {0,};
  FILE *f;
  int ret;
  int ch;
  char *me = argv[0];
  unsigned long n = 0;
  int display_extension = 0;

  while((ch = getopt(argc, argv, "hFRESVn:x")) != -1) {
    switch(ch) {
    case 'F': allow_fileinto = 0; break;
    case 'R': allow_reject = 0; break;
    case 'E': allow_enveloppe = 0; break;
    case 'S': allow_spamtest = 0; break;
    case 'V': allow_virustest = 0; break;
    case 'A': allow_subadress = 0; break;
    case 'n': n = strtoul(optarg, NULL, 10); break;
    case 'x': display_extension = 1; break;
    case '?':
    case 'h':
    default:
      fprintf(stderr, "usage %s: [-F -R -E -S -V -A -x] [ -n x ] [file]\n", me);
      fprintf(stderr, "          -F    disable fileinto\n");
      fprintf(stderr, "          -V    disable reject\n");
      fprintf(stderr, "          -E    disable envelope\n");
      fprintf(stderr, "          -S    disable spamtest\n");
      fprintf(stderr, "          -V    disable virustest\n");
      fprintf(stderr, "          -A    disable subadress\n");
      fprintf(stderr, "          -n X  make the check X times\n");
      fprintf(stderr, "          -x    display supported extensions and exit\n");
      exit(1);
    }
  }

  argc -= optind;
  argv += optind;

  if (allow_fileinto) {
    Pmethods.valid_folder = valid_folder;
  }
  
  if (allow_subadress) {
    Pmethods.separator = '+';
  }

  if (allow_enveloppe) {
    Pmethods.from = from;
    Pmethods.to = to;
  }

  Pmethods.allow_reject = allow_reject;
  Pmethods.size = size;

  if (allow_virustest) 
    Pmethods.virus = virustest;

  if (allow_spamtest) 
    Pmethods.spam = spamtest;

  if (argc >= 1) {
    f = fopen(argv[0], "r"); 
    if (f == NULL) {
      perror(argv[0]);
      exit(2);
    }
  } else {
    f = stdin;
  }


  if (display_extension) {
    char **ext = sieve_extensions(&Pmethods);
    while(*ext) {
      printf("%s\n", *ext++);
    }
    exit(0);
  }

  do {
    ret = sieve_check((sieve_fgetc_t)fgetc, f, &Pmethods, NULL, &Presult);
    if (Presult.error[0]) {
      fprintf(stderr, "%s near line %d\n", Presult.error, Presult.lastline);
    }
    //    sieve_result_free(result);
    rewind(f);
  } while (n-->0);
  return ret;
}
