
#include <stddef.h>

#define SIEVE_STRING_LEN 1000

typedef int (*sieve_fgetc_t)(void *);

struct sieve_methods {
  char *(*header)(void *message, const char *header,
		  int decoded, void **state);
  void (*header_state_free)(void *message, void *state);
  size_t (*size)(void *message);
  unsigned (*spam)(void *message);
  unsigned (*virus)(void *message);
  char *(*from)(void *message); /* enveloppe from */
  char *(*to)(void *message);   /* enveloppe to */
  int allow_reject;
  /* return 1 if the candidate is a valid folder with fileinto */
  /* set to NULL if fileinto is disallowed */
  int (*valid_folder)(void *message, const char *candidate); 
  char separator;
};

struct sieve_result {
  int implicit_keep;
  char **redirect;
  char **fileinto;
  char *reject;
  char error[SIEVE_STRING_LEN];
  int lastline;
};


/*
 * returns
 */

int sieve(sieve_fgetc_t sieve_fgetc, void *file,
	  struct sieve_methods *methods, void *message,
	  struct sieve_result *result);

int sieve_check(sieve_fgetc_t sieve_fgetc, void *file,
		struct sieve_methods *methods, void *message,
		struct sieve_result *result);

/* free a struct sieve_result */
void sieve_result_free(struct sieve_result *result);

/* return a string array of allowed extensions */
/* clear the result with a single free() */
char **sieve_extensions(struct sieve_methods *methods);
