/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED =
"$Id: user_res.c,v 1.7 2004/10/26 09:59:20 lwa Exp $";



#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/nameser.h>
#include <resolv.h>
#include <syslog.h>

#if !HAVE_DECL_RES_INIT
int res_init(void); /* missing declaration on HPUX */
#endif

#include "lmtp.h"

/*
 * setup and restore environment for resolving
 */

int user_res_begin(struct fstate *state, struct user_res_ctx *ctx) {
  if ((_res.options & RES_INIT) == 0 && res_init() <0) {
    syslog(LOG_ERR, "DNS initialization failure");
    return -1;
  }
  _res.options &= ~(RES_DEBUG | RES_DNSRCH | RES_DEFNAMES);
  _res.options |= RES_NOALIASES;
  if (state->res_retry) {
    ctx->res_retry=_res.retry;
    _res.retry = state->res_retry;
  }
  if (state->res_retrans) {
    ctx->res_retrans=_res.retrans;
    _res.retrans=state->res_retrans;
  }
  return 0;
}

int user_res_end(struct fstate *state, struct user_res_ctx *ctx) {
  if (state->res_retrans)
    _res.retrans = ctx->res_retrans;
  if (state->res_retry)
    _res.retry = ctx->res_retry;
  return 0;
}
